#	INPUT.R

#	GAL FILES

readGAL <- function(galfile=NULL,path=NULL,header=TRUE,sep="\t",quote="\"",skip=NULL,as.is=TRUE,...) {
#	Read GenePix Allocation List (GAL) file
#	Gordon Smyth
#	1 Mar 2003.  Last revised 16 Sept 2003.

	if(is.null(galfile)) {
		if(is.null(path)) path <- "."
		galfile <- dir(path=path,pattern="*\\.gal$")
		nfiles <- length(galfile)
		if(nfiles == 0) stop("Cannot find GAL file")
		if(nfiles > 1) {
			galfile <- galfile[1]
			warning(paste("More than one GAL file found. Reading",galfile))
		}
	}
	if(!is.null(path)) galfile <- file.path(path,galfile)
	if(is.null(skip)) {
		chunk <- readLines(galfile,n=100)
		skip <- intersect(grep("Name",chunk), grep("ID",chunk)) - 1
		n <- length(skip)
		if(n == 0) stop("Cannot find ID and Name columns in GAL file")
		if(n > 1) stop("Multiple lines with ID and Name labels")
	}
	read.table(galfile,header=header,sep=sep,quote=quote,skip=skip,as.is=as.is,comment.char="",...)
}

splitName <- function(x, split=";", extended=TRUE) {
#	Split composite gene names into short names and annotation information
#	Gordon Smyth
#	8 May 2003.  Last modified 29 May 2003.

	s <- strsplit(x,split,extended)
	function1 <- function(x) {
		n <- length(x)
		if(n > 0)
			if(n > 2) paste(x[1:(n-1)],collapse=split) else x[1]
		else
			""
	}
	function2 <- function(x) {
		n <- length(x)
		if(n > 1) x[n] else ""
	}
	list(Name=unlist(lapply(s,function1)), Annotation=unlist(lapply(s,function2)))
}

getLayout <- function(gal) {
#	Guess print layout from a GenePix Allocation List (GAL)
#	Gordon Smyth
#	7 Apr 2003.  Last revised 8 May 2003.

	n <- nrow(gal)
	lapply(list(ngrid.r=gal$Block[n]/4,ngrid.c=4,nspot.r=gal$Row[n],nspot.c=gal$Column[n]),as.integer)
}

readTargets <- function(file="Targets.txt",sep="\t")
#	Data frame of target information
#	Gordon Smyth
#	19 Oct 2003
{
	tab <- read.table(file,header=TRUE,as.is=TRUE,sep=sep,quote="\"")
	if(all(c("Cy3","Cy5") %in% names(tab)))
		return(tab)
	else
		stop("File should contain columns: Cy3 and Cy5")
}

readSpotTypes <- function(file="SpotTypes.txt",sep="\t")
#	Read regexp for spot types
#	Gordon Smyth following idea of James Wettenhall
#	19 Oct 2003
{
	tab <- read.table(file,header=TRUE,as.is=TRUE,sep=sep,quote="\"")
	if(all(c("SpotType","ID","Name","Color") %in% names(tab)))
		return(tab)
	else
		stop("File should contain columns: SpotType, ID, Name and Color")
}

controlStatus <- function(types, genes)
#	Set status of each spot
#	Gordon Smyth
#	19 Oct 2003.  Last modified 21 Oct 2003.
{
	if(is(genes,"RGList") || is(genes,"MAList")) genes <- genes$genes
	status <- character(nrow(genes))
	for (i in 1:nrow(types)) {
		sel <- intersect(grep(types$ID[i],genes$ID),grep(types$Name[i],genes$Name))
		status[sel] <- types$SpotType[i]
	}
	status
}

#	READ IMAGE ANALYSIS FILES INTO RG LIST

read.matrix <- function(file,nrows=0,skip=0,...) {
#	Read numeric matrix with headers from file
#	Gordon Smyth
#	9 Mar 2003.

	h <- scan(file,what="character",skip=skip,nlines=1,quote="\"",quiet=TRUE,...)
	x <- matrix(scan(file,skip=skip+1,nlines=nrows,quiet=TRUE,...),byrow=TRUE,ncol=length(h))
	colnames(x) <- h
	x
}

read.maimages <- function(files,source="spot",path=NULL,ext=NULL,names=NULL,columns=NULL,wt.fun=NULL,verbose=TRUE,sep="\t",quote="\"",...) {
#	Extracts an RG list from a series of image analysis output files
#	Gordon Smyth
#	1 Nov 2002.  Last revised 27 Sep 2003.

	if(missing(files)) {
		if(missing(ext))
			stop("Must specify input files")
		else {
			extregex <- paste("\\.",ext,"$",sep="")
			files <- dir(path=ifelse(is.null(path),".",path),pattern=extregex)
			files <- sub(extregex,"",files)
		}
	}
	if(!missing(source) && !missing(columns)) stop("Cannot specify both source and columns")
	source <- match.arg(source,c("arrayvision","genepix","imagene","quantarray","smd","spot","spot.close.open"))
	if(source=="imagene") return(read.imagene(files=files,path=path,ext=ext,names=names,columns=columns,wt.fun=wt.fun,verbose=verbose,sep=sep,quote=quote,...))
	slides <- as.vector(as.character(files))
	if(!is.null(ext)) slides <- paste(slides,ext,sep=".")
	nslides <- length(slides)
	if(is.null(names)) names <- removeExt(files)

	if(is.null(columns)) columns <- switch(source,
		smd = list(Gf="CH1I_MEAN",Gb="CH1B_MEDIAN",Rf="CH2I_MEAN",Rb="CH2B_MEDIAN"),
		spot = list(Rf="Rmean",Gf="Gmean",Rb="morphR",Gb="morphG"),
		spot.close.open = list(Rf="Rmean",Gf="Gmean",Rb="morphR.close.open",Gb="morphG.close.open"),
		genepix = list(Rf="F635 Mean",Gf="F532 Mean",Rb="B635 Median",Gb="B532 Median"),
		quantarray = list(Rf="ch2 Intensity",Gf="ch1 Intensity",Rb="ch2 Background",Gb="ch1 Background")
	)

#	Read first file to get nspots
	fullname <- slides[1]
	if(!is.null(path)) fullname <- file.path(path,fullname)
	if(source=="quantarray") {
		firstfield <- scan(fullname,what="",sep="\t",flush=TRUE,quiet=TRUE,blank.lines.skip=FALSE,multi.line=FALSE)
		skip <- grep("Begin Data",firstfield)
		if(length(skip)==0) stop("Cannot find \"Begin Data\" in image output file")
		nspots <- grep("End Data",firstfield) - skip -2
		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char="",nrows=nspots,...)
	} else if(source=="arrayvision") {
		skip <- 1
		cn <- scan(fullname,what="",sep=sep,quote=quote,skip=1,nlines=1,quiet=TRUE)
		fg <- grep("^Median Dens - RFU",cn)
		if(length(fg) != 2) stop(paste("Cannot find foreground columns in",fullname))
		bg <- grep("Bkgd",cn)
		if(length(fg) != 2) stop(paste("Cannot find background columns in",fullname))
		columns <- list(Rf=fg[1],Rb=bg[1],Gf=fg[2],Gb=bg[2])
		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char="",...)
		nspots <- nrow(obj)
	} else {
		skip <- grep(columns$Rf,readLines(fullname,n=80)) - 1
		if(length(skip)==0)
			stop("Cannot find column heading in image output file")
		else
			skip <- skip[1]
		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char="",...)
		nspots <- nrow(obj)
	}

#	Now read rest
	Y <- matrix(0,nspots,nslides)
	colnames(Y) <- names
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y)
	if(source=="smd") {
		anncol <- grep(columns$Gf,colnames(obj))-1
		if(anncol>0) RG$genes <- data.frame(obj[,1:anncol])
	}
	if(!is.null(wt.fun)) RG$weights <- Y
	for (i in 1:nslides) {
		if(i > 1) {
			fullname <- slides[i]
			if(!is.null(path)) fullname <- file.path(path,fullname)
			obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,as.is=TRUE,quote=quote,check.names=FALSE,comment.char="",nrows=nspots,...)
		}
		RG$R[,i] <- obj[,columns$Rf]
		RG$G[,i] <- obj[,columns$Gf]
		RG$Rb[,i] <- obj[,columns$Rb]
		RG$Gb[,i] <- obj[,columns$Gb]
		if(!is.null(wt.fun)) RG$weights[,i] <- wt.fun(obj)
		if(verbose) cat(paste("Read",fullname,"\n"))
	}
	new("RGList",RG)
}

read.imagene <- function(files,path=NULL,ext=NULL,names=NULL,columns=NULL,wt.fun=NULL,verbose=TRUE,sep="\t",quote="\"",...) {
#	Extracts an RG list from a series of Imagene analysis output files.
#	Imagene requires special treatment because red and green channel
#	intensities are in different files.
#	Gordon Smyth
#	14 Aug 2003.  Last modified 2 Oct 2003.

	files <- as.matrix(files)
	if(ncol(files) != 2) stop("Need a two column matrix of file names")
	if(!is.null(ext)) files <- array(paste(files,ext,sep="."),dim(files))
	narrays <- nrow(files)
	if(is.null(columns)) columns <- list(f="Signal Mean",b="Background Median")
	if(is.null(columns$f) || is.null(columns$b)) stop("'columns' should have components 'f' and 'b'")
	if(is.null(names)) names <- removeExt(files[,1])

#	Read header information from first file to get nspots
	fullname <- files[1,1]
	if(!is.null(path)) fullname <- file.path(path,fullname)
	headers <- readImageneHeaders(fullname)
	if(verbose) cat("Read header information\n")
	skip <- headers$Begin.Raw.Data
	printer <- headers$Field.Dimensions[c("Metarows","Metacols","Rows","Cols")]
	names(printer) <- c("ngrid.r","ngrid.c","nspot.r","nspot.c")
	if(length(printer) != 4) stop("Cannot read field dimension information")
	nspots <- prod(unlist(printer))

#	Now read data
	Y <- matrix(0,nspots,narrays)
	colnames(Y) <- names
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y,printer=printer)
	if(!is.null(wt.fun)) RG$weights <- Y
	for (i in 1:narrays) {
		fullname <- files[i,1]
		if(!is.null(path)) fullname <- file.path(path,fullname)
		if(i > 1) {
			headers <- readImageneHeaders(fullname)
			if(any(unlist(printer) != unlist(headers$Field.Dimensions[c("Metarows","Metacols","Rows","Cols")])))
				stop(paste("Field dimensions of array",i,"not same as those of first array"))
			skip <- headers$Begin.Raw.Data
		}
		obj<- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,check.names=FALSE,comment.char="",nrows=nspots,...)
		if(verbose) cat(paste("Read",fullname,"\n"))
		if(i==1) RG$genes <- obj[,c("Field","Meta Row","Meta Column","Row","Column","Gene ID")]
		RG$G[,i] <- obj[,columns$f]
		RG$Gb[,i] <- obj[,columns$b]
		fullname <- files[i,2]
		if(!is.null(path)) fullname <- file.path(path,fullname)
		obj<- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,check.names=FALSE,comment.char="",nrows=nspots,...)
		if(verbose) cat(paste("Read",fullname,"\n"))
		RG$R[,i] <- obj[,columns$f]
		RG$Rb[,i] <- obj[,columns$b]
		if(!is.null(wt.fun)) RG$weights[,i] <- wt.fun(obj)
	}
	new("RGList",RG)
}

readGPRHeaders <- function(file) {
#	Extracts header information from a GenePix Results (GPR) file
#	Gordon Smyth
#	4 October 2003.

	con <- file(file, "r")	
	if(substring(readLines(con,n=1),1,3) != "ATF") stop("File is not in Axon Text File (ATF) format")
	nfields <- as.numeric(strsplit(readLines(con,n=1),split="\t")[[1]])
	out <- list()
	for (i in 1:nfields[1]) {
		txt <- strsplit(sub("\"$","",sub("^\"","",readLines(con,n=1))),split="=")[[1]]
		out[[i]] <- txt[2]
		names(out)[i] <- txt[1]
	}
	close(con)
	out$Wavelengths <- strsplit(out$Wavelengths,split="\t")[[1]]
	out$ImageFiles <- strsplit(out$ImageFiles,split="\t")[[1]]
	out
}

readImageneHeaders <- function(file) {
#	Extracts header information from an Imagene analysis output file
#	Gordon Smyth
#	14 Aug 2003.

	firstfield <- scan(file,what="",sep="\t",quote="\"",nlines=60,flush=TRUE,quiet=TRUE,blank.lines.skip=FALSE,multi.line=FALSE)
	beginrawdata <- grep("Begin Raw Data",firstfield)
	txt <- scan(file,what="",sep="\t",quote="\"",nlines=beginrawdata-1,quiet=TRUE)
	out <- list(Begin.Raw.Data=beginrawdata)
	out$Version <- txt[grep("^version$",txt)+1]
	out$Date <- txt[grep("^Date$",txt)+1]
	out$Image.File <- txt[grep("^Image File$",txt)+1]
	out$Inverted <- as.logical(txt[grep("^Inverted$",txt)+1])
	out$Field.Dimensions <- list()
	out$Field.Dimensions$Field <- txt[grep("^Field$",txt)+7]
	out$Field.Dimensions$Metarows <- as.integer(txt[grep("^Metarows$",txt)+7])
	out$Field.Dimensions$Metacols <- as.integer(txt[grep("^Metacols$",txt)+7])
	out$Field.Dimensions$Rows <- as.integer(txt[grep("^Rows$",txt)+7])
	out$Field.Dimensions$Cols <- as.integer(txt[grep("^Cols$",txt)+7])
	out$Measurement.Parameters <- list()
	out$Measurement.Parameters$Signal.Low <- txt[grep("^Signal Low$",txt)+1]
	out$Measurement.Parameters$Signal.High <- txt[grep("^Signal High$",txt)+1]
	out$Measurement.Parameters$Background.Low <- txt[grep("^Background Low$",txt)+1]
	out$Measurement.Parameters$Background.High <- txt[grep("^Background High$",txt)+1]
	out$Measurement.Parameters$Background.Buffer <- txt[grep("^Background Buffer$",txt)+1]
	out$Measurement.Parameters$Background.Width <- txt[grep("^Background Width$",txt)+1]
	out
}

rg.series.spot <- function(slides,path=NULL,names.slides=names(slides),suffix="spot",wt.fun=NULL,verbose=TRUE,...) {
#	Extracts an RG list from a series of Spot image analysis files
#	Gordon Smyth
#	1 Nov 2002.  Last revised 23 Mar 2003.

	slides <- as.vector(as.character(slides))
	if(is.null(names.slides)) names.slides <- slides
	nslides <- length(slides)
	if(!is.null(suffix)) slides <- paste(slides,suffix,sep=".")

#	Read first file to get nspots
	fullname <- slides[1]
	if(!is.null(path)) fullname <- file.path(path,fullname)
	obj <- read.matrix(fullname,...)
	nspots <- nrow(obj)

#	Now read rest
	Y <- matrix(0,nspots,nslides)
	colnames(Y) <- names.slides
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y)
	if(!is.null(wt.fun)) RG$weights <- Y
	for (i in 1:nslides) {
		if(i > 1) {
			fullname <- slides[i]
			if(!is.null(path)) fullname <- file.path(path,fullname)
			obj <- read.matrix(fullname,nrows=nspots,...)
		}
		RG$R[,i] <- obj[,"Rmean"]
		RG$G[,i] <- obj[,"Gmean"]
		RG$Rb[,i] <- obj[,"morphR"]
		RG$Gb[,i] <- obj[,"morphG"]
		if(!is.null(wt.fun)) RG$weights[,i] <- wt.fun(obj)
		if(verbose && interactive()) cat(paste("Read",fullname,"\n"))
	}
	RG
}

#	READ COMPLETE IMAGE ANALYSIS OUTPUT FILES INTO DATA FRAMES

read.series <- function(slides,path=NULL,suffix="spot",...) {
#	Read in a series of image analysis files
#	Gordon Smyth
#	11 Mar 2002.  Last revised 2 Mar 2003.

	slides <- as.vector(as.character(slides))
	nslides <- length(slides)
	for (i in 1:nslides) {
		slidename <- slides[i]
		if(!is.null(suffix)) slidename <- paste(slidename,suffix,sep=".")
		fullname <- slidename
		if(!is.null(path)) fullname <- file.path(path,slidename)
		assign(slidename,read.table(fullname,header=TRUE,...),pos=1)
	}
	return(paste(as.character(nslides),"slides read"))
}

#	EXTRACT COLUMNS OF INTEREST FROM COMPLETE DATA FRAMES

m.spot <- function(spot) {
#	Extracts log-ratio M from a Spot image analysis dataframe
#	Gordon Smyth
#	18 Nov 2001.  Last revised 2 Mar 2003.

	R <- spot[,"Rmean"] - spot[,"morphR"]
	G <- spot[,"Gmean"] - spot[,"morphG"]
	log(R,2) - log(G,2)
}

a.spot <- function(spot) {
#	Extracts intensity A from a Spot image analysis dataframe
#	Gordon Smyth
#	18 Nov 2001.  Last revised 2 Mar 2003.

	R <- spot[,"Rmean"] - spot[,"morphR"]
	G <- spot[,"Gmean"] - spot[,"morphG"]
	(log(R,2) + log(G,2))/2
}

rg.spot <- function(slides,names.slides=names(slides),suffix="spot",area=FALSE) {
#	Extract RG list from a series of Spot dataframes
#	Gordon Smyth
#	17 Jan 2002. Last revised 1 Nov 2002.

	slides <- as.vector(as.character(slides))
	if(is.null(names.slides)) names.slides <- slides
	nslides <- length(slides)
	if(!is.null(suffix)) slides <- paste(slides,suffix,sep=".")
	ngenes <- dim(get(slides[1]))[1]
	Y <- matrix(0,ngenes,nslides)
	colnames(Y) <- names.slides
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y)
	if(area) RG$Area <- Y
	for (i in 1:nslides) {
		RG$R[,i] <- get(slides[i])[,"Rmean"]
		RG$G[,i] <- get(slides[i])[,"Gmean"]
		RG$Rb[,i] <- get(slides[i])[,"morphR"]
		RG$Gb[,i] <- get(slides[i])[,"morphG"]
		if(area) RG$Area[,i] <- get(slides[i])[,"area"]
	}
	RG
}

rg.quantarray <- function(slides,names.slides=names(slides),suffix="qta") {
#	Extract RG list from a series of Quantarray dataframes
#	Gordon Smyth
#	23 July 2002

	slides <- as.vector(as.character(slides))
	if(is.null(names.slides)) names.slides <- slides
	nslides <- length(slides)
	if(!is.null(suffix)) for (i in 1:nslides) slides[i] <- paste(slides[i],suffix,sep=".")
	ngenes <- dim(get(slides[1]))[1]
	Y <- matrix(0,ngenes,nslides)
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y)
	for (i in 1:nslides) {
		RG$R[,i] <- get(slides[i])$ch2.Intensity
		RG$G[,i] <- get(slides[i])$ch1.Intensity
		RG$Rb[,i] <- get(slides[i])$ch2.Background
		RG$Gb[,i] <- get(slides[i])$ch1.Background
	}
	colnames(RG$R) <- colnames(RG$G) <- colnames(RG$Rb) <- colnames(RG$Gb) <- names.slides
	RG
}

rg.genepix <- function(slides,names.slides=names(slides),suffix="gpr") {
#	Extract RG list from a series of Genepix dataframes
#	Gordon Smyth
#	23 July 2002. Last revised 13 Feb 2003.

	slides <- as.vector(as.character(slides))
	if(is.null(names.slides)) names.slides <- slides
	nslides <- length(slides)
	if(!is.null(suffix)) for (i in 1:nslides) slides[i] <- paste(slides[i],suffix,sep=".")
	ngenes <- dim(get(slides[1]))[1]
	Y <- matrix(0,ngenes,nslides)
	RG <- list(R=Y,G=Y,Rb=Y,Gb=Y)
	for (i in 1:nslides) {
		RG$R[,i] <- get(slides[i])$F635.Mean
		RG$G[,i] <- get(slides[i])$F532.Mean
		RG$Rb[,i] <- get(slides[i])$B635.Median
		RG$Gb[,i] <- get(slides[i])$B532.Median
	}
	colnames(RG$R) <- colnames(RG$G) <- colnames(RG$Rb) <- colnames(RG$Gb) <- names.slides
	RG
}

removeExt <- function(x) {
#	Remove any common extension from a vector of file names
#	Gordon Smyth
#	19 July 2002.

	n <- length(x)
	if(length(grep("\\.",x)) < n) return(x)
	ext <- sub("(.*)\\.(.*)$","\\2",x)
	if(all(ext[1] == ext))
		return(sub("(.*)\\.(.*)$","\\1",x))
	else
		return(x)
}

#	LAYOUT FUNCTIONS

spotc <- function(layout) {
#	GKS  24 Nov 2002
	rep(1:layout$nspot.c, length=prod(unlist(layout)))
}

spotr <- function(layout) {
#	GKS  24 Nov 2002
	rep(1:layout$nspot.r, times=layout$ngrid.r*layout$ngrid.c, each=layout$nspot.c)
}

gridc <- function(layout) {
#	GKS  24 Nov 2002
	rep(1:layout$ngrid.c, times=layout$ngrid.r, each=layout$nspot.c*layout$nspot.r)
}

gridr <- function(layout) {
#	GKS  24 Nov 2002
	rep(1:layout$ngrid.r, each=layout$nspot.c*layout$nspot.r*layout$ngrid.c)
}

printorder <- function(layout, ndups=1, npins=layout$ngrid.r*layout$ngrid.c, start="topleft") {
#	Identify order in which spots were printed
#	Gordon Smyth
#	2 May 2003.  Last revised 2 July 2003.

	if(is(layout,"RGList") || is(layout,"MAList")) {
		if(missing(ndups) && !is.null(layout$printer$ndups)) ndups <- layout$printer$ndups
		if(missing(npins) && !is.null(layout$printer$npins)) npins <- layout$printer$npins
		if(missing(start) && !is.null(layout$printer$start)) start <- layout$printer$start
		layout <- layout$printer
	}

#	All input values should be integers
#	DNA plates assumed to have 384 wells
	nwell.r <- as.integer(16)
	nwell.c <- as.integer(24)
	nwells <- nwell.r * nwell.c
	ngrid.r <- layout$ngrid.r
	ngrid.c <- layout$ngrid.c
	nspot.r <- layout$nspot.r
	nspot.c <- layout$nspot.c
#	Number of grid blocks
	ngrids <- ngrid.r * ngrid.c
#	Number of spots per grid
	nspots <- nspot.r * nspot.c
	ndups <- round(ndups)
	npins <- round(npins)
#	Pin columns assumed to be same an grid columns
	npin.c <- ngrid.c
	npin.r <- npins %/% npin.c
#	Number of times print head dips into each plate
	ndips <- ndups * nwells %/% npins

#	First do everything for subarray with ngrids=npins

	spot.c <- rep(1:nspot.c, times = npins * nspot.r)
	spot.r <- rep(1:nspot.r, times = npins, each = nspot.c)
	pin.c <- rep(1:npin.c, times = npin.r, each = nspots)
	pin.r <- rep(1:npin.r, each = npin.c * nspots)

#	Printorder
	start <- match.arg(start,c("topright","topleft"))
	po <- switch(start,
		topleft=nspot.c*(spot.r-1)+spot.c,
		topright=nspot.c*(spot.r-1)+nspot.c-spot.c+1
	)

#	Now repeat subarrays if fewer pins than actual grids

	if(ngrids %% npins != 0) stop("ngrids not a multiple of npins")
	m <- ngrids %/% npins
	if(m > 1) po <- rep(po,m) + nspots * rep(0:(m-1), each = npins * nspots)

#	Plates
	plate <- 1 + (po-1) %/% ndips

#	Position within plate
	platedip <- 1 + (po-1) %% ndips
	plateblock <- 1 + (platedip-1) %/% ndups
	plateblock.r <- 1 + (plateblock-1) %/% (nwell.c %/% npin.r)
	plateblock.c <- 1 + (plateblock-1) %% (nwell.c %/% npin.r)
	plate.r <- (plateblock.r-1) * npin.c + npin.c - pin.c + 1
	plate.c <- (plateblock.c-1) * npin.r + pin.r
	platedigits <- 1+floor(log(ngrids*nspots/ndups/nwells,10))
	platepos <- paste("p",formatC(plate, width=platedigits, flag = "0"), LETTERS[plate.r], formatC(plate.c, width=2, flag = "0"), sep="")

	list(printorder=po, plate=plate, plate.r=plate.r, plate.c=plate.c, plateposition=platepos)
}

